# call this via "python[3] script name"
import sys
from pysnmp.entity import engine, config
from pysnmp.carrier.asyncore.dgram import udp
from pysnmp.entity.rfc3413 import ntfrcv
from pysnmp.smi import builder, view, compiler, rfc1902
from pyasn1.type.univ import OctetString

# Global variables
snmpport = 10162
snmpip = "127.0.0.1"
szOutputfile = "snmp.out"
szSnmpLogfile = "snmp_server.log"

# For vrebose output
bDebug = False

# Read command line params
if len(sys.argv) > 1:
    snmpport = int(sys.argv[1])
if len(sys.argv) > 2:
    snmpip = sys.argv[2]
if len(sys.argv) > 3:
    szOutputfile = sys.argv[3]
if len(sys.argv) > 4:
    szSnmpLogfile = sys.argv[4]

# Create output files
outputFile = open(szOutputfile,"w+")
logFile = open(szSnmpLogfile,"a+")

# Assemble MIB viewer
mibBuilder = builder.MibBuilder()
compiler.addMibCompiler(mibBuilder, sources=['file:///usr/share/snmp/mibs', 'file:///var/lib/snmp/mibs', '/usr/local/share/snmp/mibs/'])
mibViewController = view.MibViewController(mibBuilder)
# Pre-load MIB modules we expect to work with
try:
    mibBuilder.loadModules('SNMPv2-MIB', 'SNMP-COMMUNITY-MIB', 'SYSLOG-MSG-MIB')
except Exception:
    print("Failed loading MIBs")

# Create SNMP engine with autogenernated engineID and pre-bound to socket transport dispatcher
snmpEngine = engine.SnmpEngine()

# Transport setup
# UDP over IPv4, add listening interface/port
config.addTransport(
    snmpEngine,
    udp.domainName + (1,),
    udp.UdpTransport().openServerMode((snmpip, snmpport))
)

# SNMPv1/2c setup
# SecurityName <-> CommunityName mapping
config.addV1System(snmpEngine, 'my-area', 'public')

print("Started SNMP Trap Receiver: %s, %s, Output: %s" % (snmpport, snmpip, szOutputfile))
logFile.write("Started SNMP Trap Receiver: %s, %s, Output: %s" % (snmpport, snmpip, szOutputfile))
logFile.flush()

# Add PID file creation after startup message
import os
with open(szSnmpLogfile + ".started", "w") as f:
    f.write(str(os.getpid()))

# Callback function for receiving notifications
# noinspection PyUnusedLocal,PyUnusedLocal,PyUnusedLocal
def cbReceiverSnmp(snmpEngine, stateReference, contextEngineId, contextName, varBinds, cbCtx):
    transportDomain, transportAddress = snmpEngine.msgAndPduDsp.getTransportInfo(stateReference)
    if (bDebug):
        szDebug = str("Notification From: %s, Domain: %s, SNMP Engine: %s, Context: %s" %
            (transportAddress, transportDomain, contextEngineId.prettyPrint(), contextName.prettyPrint()))
        print(szDebug)
        logFile.write(szDebug)
        logFile.flush()

    # Create output String
    szOut = "Trap Source{}, Trap OID {}".format(transportAddress, transportDomain)

    varBinds = [rfc1902.ObjectType(rfc1902.ObjectIdentity(x[0]), x[1]).resolveWithMib(mibViewController) for x in varBinds]

    for name, val in varBinds:
        # Append to output String
        szOut = szOut + ", Oid: {}, Value: {}".format(name.prettyPrint(), val.prettyPrint())

        if isinstance(val, OctetString):
            if (name.prettyPrint() != "SNMP-COMMUNITY-MIB::snmpTrapAddress.0"):
                szOctets = val.asOctets()#.rstrip('\r').rstrip('\n')
                szOut = szOut + ", Octets: {}".format(szOctets)
        if (bDebug):
            print('%s = %s' % (name.prettyPrint(), val.prettyPrint()))
    outputFile.write(szOut)
    if "\n" not in szOut:
        outputFile.write("\n")
    outputFile.flush()


# Register SNMP Application at the SNMP engine
ntfrcv.NotificationReceiver(snmpEngine, cbReceiverSnmp)

# this job would never finish
snmpEngine.transportDispatcher.jobStarted(1)

# Run I/O dispatcher which would receive queries and send confirmations
try:
    snmpEngine.transportDispatcher.runDispatcher()
except:
    os.remove(szOutputfile + ".started")  # Remove PID file on shutdown
    snmpEngine.transportDispatcher.closeDispatcher()
    raise


# call this via "python[3] script name"
# a small url encoder for testbench purposes
# written 2018-11-05 by Rainer Gerhards
# part of the rsyslog testbench, released under ASL 2.0
import sys
try:
    from urllib.parse import quote_plus
except ImportError:
    from urllib import quote_plus

if len(sys.argv) != 2:
    print("ERROR: urlencode needs exactly one string as argument")
    sys.exit(1)
print(quote_plus(sys.argv[1]))



# source/_ext/rsyslog_lexer.py

import re

from pygments.lexer import RegexLexer, bygroups
from pygments.token import (
    Text, Comment, Operator, Keyword, Name, String, Number, Punctuation,
    Error
)

__all__ = ['RainerScriptLexer']


class RainerScriptLexer(RegexLexer):
    """
    Pygments lexer for Rsyslog's RainerScript.
    """
    name = 'RainerScript'
    aliases = ['rsyslog', 'rainerscript']
    filenames = ['*.conf', '*.rs', '*.rainerscript']
    mimetypes = ['text/x-rainerscript']

    # Keywords from the lexer file
    keywords = [
        'if', 'then', 'else', 'call', 'call_indirect', 'set', 'unset',
        'reset', 'stop', 'continue', 'foreach', 'do', 'return'
    ]

    # Object definition keywords
    objects = [
        'action', 'input', 'module', 'template', 'ruleset', 'parser',
        'global', 'property', 'constant', 'lookup_table', 'main_queue',
        'dyn_stats', 'percentile_stats', 'timezone', 'include',
        'reload_lookup_table'
    ]
    
    # Built-in functions and system properties
    builtins = [
        'exec_template', 'lookup', 're_match', 're_extract', 'field', 'dyn_inc',
        'http_request', 'getenv', 'uuid', 'random', 'get_metadata',
        'set_metadata', 'unset_metadata', 'exists', 'int', 'string',
        'get_var', 'hostname', 'ip', 'programname', 'msg', 'rawmsg',
        'syslogtag', 'protocol-version', 'structured-data', 'app-name',
        'procid', 'msgid', 'pri', 'pri-text', 'syslogfacility',
        'syslogfacility-text', 'syslogseverity', 'syslogseverity-text',
        'fromhost', 'fromhost-ip', 'fromhost-port', 'remotehost', 'remotehost-ip', 'timereported',
        'timegenerated', 'timestamp', 'json', 'jsonmesg', 'jsonf'
    ]

    tokens = {
        'root': [
            (r'\s+', Text),
            (r'#.*?\n', Comment.Single),
            (r'/\*', Comment.Multiline, 'comment'),
            
            # Legacy syslog selectors at the start of a line
            (r'^\s*([a-zA-Z0-9\.\*]+;)?([a-zA-Z0-9\.\*]+)\.(=|!=|=,!=)?([a-zA-Z0-9\*]+)\s+',
             bygroups(Keyword.Namespace, Keyword.Namespace, Operator, Keyword.Namespace), 'legacy_action'),
            (r'^\s*:([a-zA-Z0-9_-]+),\s*(==|!=|<>|contains|startswith|endswith|re_match|re_extract)\s*"[^"]*"\s+',
             bygroups(Name.Tag, Operator.Word, String.Double), 'legacy_action'),
            
            # RainerScript Keywords
            (r'\b(%s)\b' % '|'.join(keywords), Keyword.Reserved),
            (r'\b(%s)\b' % '|'.join(objects), Keyword.Declaration),
            (r'\b(on|off|yes|no|true|false)\b', Keyword.Constant),
            
            # Operators
            (r'(&&|\|\||and|or|not)\b', Operator.Word),
            (r'(=|==|!=|<>|<=|>=|<|>|:=|contains_i|startswith_i|contains|startswith|endswith)', Operator),

            # Variables and Properties
            (r'\$[a-zA-Z0-9_.-]+', Name.Variable),
            (r'\$\![\w\.-]+', Name.Variable.Instance),
            (r'\$\.[\w\.-]+', Name.Variable.Global),

            # Built-in functions
            (r'\b(%s)\b' % '|'.join(builtins), Name.Builtin),

            # Strings
            (r'"', String.Double, 'double_string'),
            (r"'", String.Single, 'single_string'),
            (r'`', String.Backtick, 'backtick_string'),

            # Numbers
            (r'\b(0x[0-9a-fA-F]+)\b', Number.Hex),
            (r'\b\d+\b', Number.Integer),
            
            # Punctuation
            (r'[{}(),;\[\].&]', Punctuation),
            
            # Identifiers
            (r'[a-zA-Z_][a-zA-Z0-9_-]*', Name), # <<<--- THIS IS THE CHANGE: Added '-' to allowed identifier characters
            
            # Legacy Directives
            (r'^\$\w+', Keyword.Pseudo),
        ],
        'comment': [
            (r'[^*/]+', Comment.Multiline),
            (r'/\*', Comment.Multiline, '#push'),
            (r'\*/', Comment.Multiline, '#pop'),
            (r'[*/]', Comment.Multiline),
        ],
        'double_string': [
            (r'\\"', String.Escape),
            (r'[^"]+', String.Double),
            (r'"', String.Double, '#pop'),
        ],
        'single_string': [
            (r"\\'", String.Escape),
            (r"[^']+", String.Single),
            (r"'", String.Single, '#pop'),
        ],
        'backtick_string': [
            (r"\\`", String.Escape),
            (r"[^`]+", String.Backtick),
            (r"`", String.Backtick, '#pop'),
        ],
        'legacy_action': [
            (r'~', Operator, '#pop'),
            (r'-?/[^;\s]+', Name.Constant, '#pop'),
            (r'\S+', Name.Constant),
            (r'.*?\n', Text, '#pop'),
        ],
    }

# --- Sphinx Extension setup function ---
def setup(app):
    from sphinx.highlighting import PygmentsBridge
    
    app.add_lexer('rainerscript', RainerScriptLexer)
    app.add_lexer('rsyslog', RainerScriptLexer) 

    return {
        'version': '0.1',
        'parallel_read_safe': True,
        'parallel_write_safe': True,
    }



# call this via "python[3] script name"
# Added 2017-11-05 by Stephen Workman, released under ASL 2.0

#
# Produces a UNIX timestamp representing the specified RFC 3164 date/time
# string. Since this date/time format does not include a year, a simple
# algorithm is used to "guess" an appropriate one and append it to the
# date/time string to calculate a timestamp value.
#
# If the incoming date is within one month in the future (from now),
# it is assumed that it's either for the current year, or the next
# year (depending on whether it is December or not).
#   - For example:
#        * If today is December 13th 2017 and we get passed the date/time
#          string "Jan  4 01:00:00", we assume that it is for the next
#          year (2018).
#        * If today is October 5th 2017, and we get passed the date/time
#          string "Nov  5 01:10:11", we assume that it is for this year.
# If the incoming date has a month "before" the current month, or does
# not fall into the situation above, it's assumed it's from the past.
#   - For example:
#        * If today is July 10th 2017, and the incoming date is for
#          a time in April, the year is assumed to be 2017.
#        * If today is July 10th 2017, and the incoming date is for
#          a time in September, the year is assumed to be 2016.
#

import re
import sys

from datetime import datetime, timedelta

err = 0

# Make tests below a little easier to read.
JAN = 1; FEB =  2; MAR =  3; APR =  4
MAY = 5; JUN =  6; JUL =  7; AUG =  8
SEP = 9; OCT = 10; NOV = 11; DEC = 12

# Run the provided expression and compare its result with the
# expected value. The function expects the expression to be
# passed in as a string so it can be printed to the screen
# as-is when there is an error.
def do_test(expr, val):
    global err

    # Run the expression and record the result
    result = eval(expr)

    # Print a message identifying the failing "test"
    if result != val:
        print("Error: %s. Expected %4d, got %4d!" % (expr, val, result))
        err += 1

# Use a sliding 12-month window (offset by one month)
# to determine the year that should be returned.
# cy - Current Year
# cm - Current Month
# im - Incoming Month
def estimate_year(cy, cm, im):
    im += 12

    if (im - cm) == 1:
        if cm == 12 and im == 13:
            return cy + 1

    if (im - cm) > 13:
        return cy - 1

    return cy

# A quick and dirty unit test to validate that our
# estimate_year() function is working as it should.
def self_test():

    # Where the incoming month is within one month
    # in the future. Should be the NEXT year if
    # the current date is in December, or the SAME
    # year if it's not December.
    do_test("estimate_year(2017, DEC, JAN)", 2018)
    do_test("estimate_year(2017, NOV, DEC)", 2017)
    do_test("estimate_year(2017, OCT, NOV)", 2017)
    do_test("estimate_year(2017, SEP, OCT)", 2017)
    do_test("estimate_year(2017, AUG, SEP)", 2017)

    # These tests validate months that are MORE than
    # one month in the future OR are before the current
    # month. If, numerically, the month comes after the
    # current month, it's assumed to be for the year
    # PRIOR, otherwise it's assumed to be from THIS year.
    do_test("estimate_year(2017, NOV, JAN)", 2017)
    do_test("estimate_year(2017, NOV, FEB)", 2017)
    do_test("estimate_year(2017, AUG, OCT)", 2016)
    do_test("estimate_year(2017, AUG, MAR)", 2017)
    do_test("estimate_year(2017, APR, JUL)", 2016)

    do_test("estimate_year(2017, AUG, JAN)", 2017)
    do_test("estimate_year(2017, APR, FEB)", 2017)

    # Additional validations based on what was described
    # above.
    do_test("estimate_year(2017, JAN, DEC)", 2016)
    do_test("estimate_year(2017, JAN, FEB)", 2017)
    do_test("estimate_year(2017, JAN, MAR)", 2016)

# Convert a datetime.timedelta object to a UNIX timestamp
def get_total_seconds(dt):
    # timedelta.total_seconds() wasn't added until
    # Python 2.7, which CentOS 6 doesn't have.

    if hasattr(timedelta, "total_seconds"):
        return dt.total_seconds()
    return dt.seconds + dt.days * 24 * 3600

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Invalid number of arguments!")
        sys.exit(1)

    if sys.argv[1] == "selftest":
        self_test()

        # Exit with non-zero if there were failures,
        # zero otherwise.
        sys.exit(err)

    months = [None, "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]

    current_datetime = datetime.utcnow()

    # The argument is expected to be an RFC 3164 timestamp
    # such as "Oct  5 01:10:11".
    incoming_datetime = sys.argv[1]

    # Get the name of the month from the date/time string that was passed in
    # and convert it to its ordinal number (1 for Jan, 10 for Oct, etc...)
    incoming_month = re.search(r"^([^ ]+) ", incoming_datetime).group(1)
    incoming_month = months.index(incoming_month)

    # Assume a year for the date/time passed in based off of today's date.
    estimated_year = estimate_year(
        current_datetime.year,
        current_datetime.month,
        incoming_month
    )

    # Convert the date/time string (now with a year, e.g. "Oct  5 01:10:11 2017") to
    # a python datetime object that we can use to calculate a UNIX timestamp
    calculated_datetime = datetime.strptime("%s %d" % (incoming_datetime, estimated_year), "%b %d %H:%M:%S %Y")

    # Convert the datetime object to a UNIX timestamp by subtracting it from the epoch
    print(int( get_total_seconds(calculated_datetime - datetime(1970,1,1)) ))



import json
import sys
import os
def checkDefaultErrorFile():
    with open(os.environ['RSYSLOG_DYNNAME'] + ".errorfile") as json_file:
        json_data = json.load(json_file)
        indexCount =0
        replyCount=0
        for item in json_data:
            if item == "request":
                for reqItem in json_data[item]:
                    if reqItem == "url":
                        print("url found")
                        print(reqItem)
                    elif reqItem == "postdata":
                        print("postdata found")
                        indexCount = str(json_data[item]).count('\"_index\":')
                        print(reqItem)
                    else:
                        print(reqItem)
                        print("Unknown item found")
                        sys.exit(1)
              
            elif item == "reply":
                for replyItem in json_data[item]:
                    if replyItem == "items":
                        print(json_data[item][replyItem])
                        replyCount = str(json_data[item][replyItem]).count('_index')
                    elif replyItem == "errors":
                        print("error node found")
                    elif replyItem == "took":
                        print("took node found")
                    else:
                        print(replyItem)
                        print("Unknown item found")
                        sys.exit(3)

            else:
                print(item)
                print("Unknown item found")
                print("error")
                sys.exit(4)
    if replyCount == indexCount :
        return 0
    else:
        sys.exit(7)
    return 0


def checkErrorOnlyFile():
    with open(os.environ['RSYSLOG_DYNNAME'] + ".errorfile") as json_file:
        json_data = json.load(json_file)
        indexCount =0
        replyCount=0
        for item in json_data:
            if item == "request":
                print(json_data[item])
                indexCount = str(json_data[item]).count('\"_index\":')
              
              
            elif item == "url":
                print("url found")
              
          
            elif item == "reply":
                print(json_data[item])
                replyCount = str(json_data[item]).count('\"_index\":')

            else:
                print(item)
                print("Unknown item found")
                print("error")
                sys.exit(4)
    if replyCount == indexCount :
        return 0
    else:
        sys.exit(7)
    return 0

def checkErrorInterleaved():
    with open(os.environ['RSYSLOG_DYNNAME'] + ".errorfile") as json_file:
        json_data = json.load(json_file)
        indexCount =0
        replyCount=0
        for item in json_data:
            print(item)
            if item == "response":
                for responseItem in json_data[item]:
                    print(responseItem)
                    for res in responseItem:
                        print(res)
                        if res == "request":
                            print(responseItem[res])
                            indexCount = str(responseItem[res]).count('\"_index\":')
                            print("request count ", indexCount)
                        elif res == "reply":
                            print(responseItem[res])
                            replyCount = str(responseItem[res]).count('\"_index\":')
                            print("reply count ", replyCount)
                        else:
                            print(res)
                            print("Unknown item found")
                            sys.exit(9)
                    if replyCount != indexCount :
                        sys.exit(8)
          
              
              
            elif item == "url":
                print("url found")
              
          
         

            else:
                print(item)
                print("Unknown item found")
                sys.exit(4)

    return 0

def checkInterleaved():
    return checkErrorInterleaved()

if __name__ == "__main__":
    option = sys.argv[1]
    if option == "default":
        checkDefaultErrorFile()
    elif option == "erroronly":
        checkErrorOnlyFile()
    elif option == "errorinterleaved":
        checkErrorInterleaved()
    elif option == "interleaved":
        checkErrorInterleaved()
    else:
        print("Usage: <script> <default|erroronly|errorinterleaved>")
        sys.exit(6)



# call this via "python[3] script name"

"""A skeleton for a python rsyslog message modification plugin
   Copyright (C) 2014 by Adiscon GmbH

   This file is part of rsyslog.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
 
         http://www.apache.org/licenses/LICENSE-2.0
         -or-
         see COPYING.ASL20 in the source distribution

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import sys
import json

# skeleton config parameters
# currently none

# App logic global variables

def onInit():
    """ Do everything that is needed to initialize processing (e.g.
        open files, create handles, connect to systems...)
    """
    # most often, nothing to do here


def onReceive(msg):
    """This is the entry point where actual work needs to be done. It receives
       the messge from rsyslog and now needs to examine it, do any processing
       necessary. The to-be-modified properties (one or many) need to be pushed
       back to stdout, in JSON format, with no interim line breaks and a line
       break at the end of the JSON. If no field is to be modified, empty
       json ("{}") needs to be emitted.
       Note that no batching takes place (contrary to the output module skeleton)
       and so each message needs to be fully processed (rsyslog will wait for the
       reply before the next message is pushed to this module).
    """
    data = json.loads(msg)
    print(json.dumps({"$!": {"sometag": "somevalue"}}))

def onExit():
    """ Do everything that is needed to finish processing (e.g.
        close files, handles, disconnect from systems...). This is
        being called immediately before exiting.
    """
    # most often, nothing to do here


"""
-------------------------------------------------------
This is plumbing that DOES NOT need to be CHANGED
-------------------------------------------------------
Implementor's note: Python seems to very agressively
buffer stdouot. The end result was that rsyslog does not
receive the script's messages in a timely manner (sometimes
even never, probably due to races). To prevent this, we
flush stdout after we have done processing. This is especially
important once we get to the point where the plugin does
two-way conversations with rsyslog. Do NOT change this!
See also: https://github.com/rsyslog/rsyslog/issues/22
"""
onInit()
keepRunning = 1
while keepRunning == 1:
    msg = sys.stdin.readline()
    if msg:
        msg = msg[:-1] # remove LF
        onReceive(msg)
        sys.stdout.flush() # very important, Python buffers far too much!
    else: # an empty line means stdin has been closed
        keepRunning = 0
onExit()
sys.stdout.flush() # very important, Python buffers far too much!



import json
import os

with open(os.environ['RSYSLOG_DYNNAME'] + ".work") as json_file:
    json_data = json.load(json_file)
    json_data = json_data["hits"]
    for item in json_data["hits"]:
        print(item["_source"]["msgnum"])



# Used by the mmkubernetes tests
# This is a simple http server which responds to kubernetes api requests
# and responds with kubernetes api server responses
# added 2018-04-06 by richm, released under ASL 2.0
import os
import json
import sys

try:
    from http.server import HTTPServer, BaseHTTPRequestHandler
except ImportError:
    from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler

ns_template = '''{{
  "kind": "Namespace",
  "apiVersion": "v1",
  "metadata": {{
    "name": "{namespace_name}",
    "selfLink": "/api/v1/namespaces/{namespace_name}",
    "uid": "{namespace_name}-id",
    "resourceVersion": "2988",
    "creationTimestamp": "2018-04-09T21:56:39Z",
    "labels": {{
      "label.1.key":"label 1 value",
      "label.2.key":"label 2 value",
      "label.with.empty.value":""
    }},
    "annotations": {{
      "k8s.io/description": "",
      "k8s.io/display-name": "",
      "k8s.io/node-selector": "",
      "k8s.io/sa.scc.mcs": "s0:c9,c4",
      "k8s.io/sa.scc.supplemental-groups": "1000080000/10000",
      "k8s.io/sa.scc.uid-range": "1000080000/10000",
      "quota.k8s.io/cluster-resource-override-enabled": "false"
    }}
  }},
  "spec": {{
    "finalizers": [
      "openshift.io/origin",
      "kubernetes"
    ]
  }},
  "status": {{
    "phase": "Active"
  }}
}}'''

pod_template = '''{{
  "kind": "Pod",
  "apiVersion": "v1",
  "metadata": {{
    "name": "{pod_name}",
    "generateName": "{pod_name}-prefix",
    "namespace": "{namespace_name}",
    "selfLink": "/api/v1/namespaces/{namespace_name}/pods/{pod_name}",
    "uid": "{pod_name}-id",
    "resourceVersion": "3486",
    "creationTimestamp": "2018-04-09T21:56:39Z",
    "labels": {{
      "component": "{pod_name}-component",
      "deployment": "{pod_name}-deployment",
      "deploymentconfig": "{pod_name}-dc",
      "custom.label": "{pod_name}-label-value",
      "label.with.empty.value":""
    }},
    "annotations": {{
      "k8s.io/deployment-config.latest-version": "1",
      "k8s.io/deployment-config.name": "{pod_name}-dc",
      "k8s.io/deployment.name": "{pod_name}-deployment",
      "k8s.io/custom.name": "custom value",
      "annotation.with.empty.value":""
    }}
  }},
  "status": {{
    "phase": "Running",
    "hostIP": "172.18.4.32",
    "podIP": "10.128.0.14",
    "startTime": "2018-04-09T21:57:39Z"
  }}
}}'''

err_template = '''{{
  "kind": "Status",
  "apiVersion": "v1",
  "metadata": {{

  }},
  "status": "Failure",
  "message": "{kind} \\\"{objectname}\\\" {err}",
  "reason": "{reason}",
  "details": {{
    "name": "{objectname}",
    "kind": "{kind}"
  }},
  "code": {code}
}}'''

is_busy = False

class SimpleHTTPRequestHandler(BaseHTTPRequestHandler):

    def do_GET(self):
        # "http://localhost:18443/api/v1/namespaces/namespace-name2"
        # parse url - either /api/v1/namespaces/$ns_name
        # or
        # /api/v1/namespaces/$ns_name/pods/$pod_name
        global is_busy
        comps = self.path.split('/')
        status = 400
        if len(comps) >= 5 and comps[1] == 'api' and comps[2] == 'v1' and comps[3] == 'namespaces':
            resp = None
            hsh = {'namespace_name':comps[4],'objectname':comps[4],'kind':'namespace'}
            if len(comps) == 5: # namespace
                resp_template = ns_template
                status = 200
            elif len(comps) == 7 and comps[5] == 'pods': # pod
                hsh['pod_name'] = comps[6]
                hsh['kind'] = 'pods'
                hsh['objectname'] = hsh['pod_name']
                resp_template = pod_template
                status = 200
            else:
                resp = '{{"error":"do not recognize {0}"}}'.format(self.path)
            if hsh['objectname'].endswith('not-found'):
                status = 404
                hsh['reason'] = 'NotFound'
                hsh['err'] = 'not found'
                resp_template = err_template
            elif hsh['objectname'].endswith('busy'):
                is_busy = not is_busy
                if is_busy:
                    status = 429
                    hsh['reason'] = 'Busy'
                    hsh['err'] = 'server is too busy'
                    resp_template = err_template
            elif hsh['objectname'].endswith('error'):
                status = 500
                hsh['reason'] = 'Error'
                hsh['err'] = 'server is failing'
                resp_template = err_template
            if not resp:
                hsh['code'] = status
                resp = resp_template.format(**hsh)
        else:
            resp = '{{"error":"do not recognize {0}"}}'.format(self.path)
        if not status == 200:
            self.log_error(resp)
        self.send_response(status)
        self.end_headers()
        self.wfile.write(json.dumps(json.loads(resp), separators=(',',':')).encode())

port = int(sys.argv[1])

httpd = HTTPServer(('localhost', port), SimpleHTTPRequestHandler)

# write "started" to file named in argv[3]
with open(sys.argv[3], "w") as ff:
    ff.write("started\n")

# write pid to file named in argv[2]
with open(sys.argv[2], "w") as ff:
    ff.write('{0}\n'.format(os.getpid()))

httpd.serve_forever()



# call this via "python[3] script name"
import argparse
import json
import os
import zlib
import base64
import random
import time

try:
    from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer # Python 2
except ImportError:
    from http.server import BaseHTTPRequestHandler, HTTPServer # Python 3

# Keep track of data received at each path
data = {}

metadata = {'posts': 0, 'fail_after': 0, 'fail_every': -1, 'decompress': False, 'userpwd': ''}


class MyHandler(BaseHTTPRequestHandler):
    """
    POST'd data is kept in the data global dict.
    Keys are the path, values are the raw received data.
    Two post requests to <host>:<port>/post/endpoint means data looks like...
        {"/post/endpoint": ["{\"msgnum\":\"00001\"}", "{\"msgnum\":\"00001\"}"]}

    GET requests return all data posted to that endpoint as a json list.
    Note that rsyslog usually sends escaped json data, so some parsing may be needed.
    A get request for <host>:<post>/post/endpoint responds with...
        ["{\"msgnum\":\"00001\"}", "{\"msgnum\":\"00001\"}"]
    """

    def validate_auth(self):
        # header format for basic authentication
        # 'Authorization: Basic <base 64 encoded uid:pwd>'
        if 'Authorization' not in self.headers:
            self.send_response(401)
            self.end_headers()
            self.wfile.write(b'missing "Authorization" header')
            return False

        auth_header = self.headers['Authorization']
        _, b64userpwd = auth_header.split()
        userpwd = base64.b64decode(b64userpwd).decode('utf-8')
        if userpwd != metadata['userpwd']:
            self.send_response(401)
            self.end_headers()
            self.wfile.write(b'invalid auth: {0}'.format(userpwd))
            return False

        return True

    def do_POST(self):
        metadata['posts'] += 1

        if metadata['userpwd']:
            if not self.validate_auth():
                return

        if metadata['fail_with_400_after'] != -1 and metadata['posts'] > metadata['fail_with_400_after']:
            if metadata['fail_with_delay_secs']:
                print("sleeping for: {0}".format(metadata['fail_with_delay_secs']))
                time.sleep(metadata['fail_with_delay_secs'])
            self.send_response(400)
            self.end_headers()
            self.wfile.write(b'BAD REQUEST')
            return

        if metadata['fail_with_401_or_403_after'] != -1 and metadata['posts'] > metadata['fail_with_401_or_403_after']:
            status = random.choice([401, 403])
            self.send_response(status)
            self.end_headers()
            self.wfile.write(b'BAD REQUEST')
            return

        if metadata['posts'] > 1 and metadata['fail_every'] != -1 and metadata['posts'] % metadata['fail_every'] == 0:
            if metadata['fail_with_delay_secs']:
                print("sleeping for: {0}".format(metadata['fail_with_delay_secs']))
                time.sleep(metadata['fail_with_delay_secs'])
            code = metadata['fail_with'] if metadata['fail_with'] else 500
            self.send_response(code)
            self.end_headers()
            self.wfile.write(b'INTERNAL ERROR')
            return

        content_length = int(self.headers['Content-Length'] or 0)
        raw_data = self.rfile.read(content_length)

        if metadata['decompress']:
            post_data = zlib.decompress(raw_data, 31)
        else:
            post_data = raw_data

        if self.path not in data:
            data[self.path] = []
        data[self.path].append(post_data.decode('utf-8'))

        res = json.dumps({'msg': 'ok'}).encode('utf8')

        self.send_response(200)
        self.send_header('Content-Type', 'application/json; charset=utf-8')
        self.send_header('Content-Length', len(res))
        self.end_headers()

        self.wfile.write(res)
        return

    def do_GET(self):
        if self.path in data:
            result = data[self.path]
        else:
            result = []

        res = json.dumps(result).encode('utf8')

        self.send_response(200)
        self.send_header('Content-Type', 'application/json; charset=utf-8')
        self.send_header('Content-Length', len(res))
        self.end_headers()

        self.wfile.write(res)
        return


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Archive and delete core app log files')
    parser.add_argument('-p', '--port', action='store', type=int, default=8080, help='port')
    parser.add_argument('--port-file', action='store', type=str, default='', help='file to store listen port number')
    parser.add_argument('-i', '--interface', action='store', type=str, default='localhost', help='port')
    parser.add_argument('--fail-after', action='store', type=int, default=0, help='start failing after n posts')
    parser.add_argument('--fail-every', action='store', type=int, default=-1, help='fail every n posts')
    parser.add_argument('--fail-with', action='store', type=int, default=500, help='on failure, fail with this code')
    parser.add_argument('--fail-with-400-after', action='store', type=int, default=-1, help='fail with 400 after n posts')
    parser.add_argument('--fail-with-401-or-403-after', action='store', type=int, default=-1, help='fail with 401 or 403 after n posts')
    parser.add_argument('--fail-with-delay-secs', action='store', type=int, default=0, help='fail with n secs of delay')
    parser.add_argument('--decompress', action='store_true', default=False, help='decompress posted data')
    parser.add_argument('--userpwd', action='store', default='', help='only accept this user:password combination')
    args = parser.parse_args()
    metadata['fail_after'] = args.fail_after
    metadata['fail_every'] = args.fail_every
    metadata['fail_with'] = args.fail_with
    metadata['fail_with_400_after'] = args.fail_with_400_after
    metadata['fail_with_401_or_403_after'] = args.fail_with_401_or_403_after
    metadata['fail_with_delay_secs'] = args.fail_with_delay_secs
    metadata['decompress'] = args.decompress
    metadata['userpwd'] = args.userpwd
    server = HTTPServer((args.interface, args.port), MyHandler)
    lstn_port = server.server_address[1]
    pid = os.getpid()
    print('starting omhttp test server at {interface}:{port} with pid {pid}'
          .format(interface=args.interface, port=lstn_port, pid=pid))
    if args.port_file != '':
        f = open(args.port_file, "w")
        f.write(str(lstn_port))
        f.close()
    server.serve_forever()



import json
import argparse
import sys
from collections import defaultdict


def parse_json_lines_to_memory(file, filename):
    """Read a file and reconstruct complete JSON objects into a list."""
    buffer = ""
    line_number = 0
    parsed_objects = []
    for line in file:
        line_number += 1
        buffer += line.strip()
        try:
            # Attempt to parse the JSON object
            parsed_objects.append(json.loads(buffer))
            buffer = ""  # Clear the buffer after successful parsing
        except json.JSONDecodeError:
            # If parsing fails, keep buffering lines
            continue
    if buffer:
        # If there's leftover data in the buffer, raise an error
        raise ValueError(
            f"Incomplete JSON object found at the end of the file '{filename}'. "
            f"Last buffered content: {buffer}"
        )
    return parsed_objects


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Validate and process log files for errors and responses.')
    parser.add_argument('--error', action='store', type=str, required=True, help='Path to the error file.')
    parser.add_argument('--response', action='store', type=str, required=True, help='Path to the response file.')
    parser.add_argument('--max-errors', action='store', type=int, default=10, help='Maximum number of errors to display.')
    args = parser.parse_args()

    messages = defaultdict(dict)
    errors = []  # Collect errors for reporting at the end

    # Load the error file into memory
    try:
        with open(args.error, "r") as error_f:
            error_data = parse_json_lines_to_memory(error_f, args.error)
            for json_obj in error_data:
                try:
                    records = json.loads(json_obj['request']['postdata'])
                    if records:
                        for i, val in enumerate(records['records']):
                            msgnum = val['value']['msgnum']
                            messages[msgnum]['response'] = json_obj['response']
                            messages[msgnum]['index'] = i
                except KeyError as e:
                    errors.append(
                        f"Missing key {e} in error file '{args.error}'. "
                        f"Problematic JSON object: {json_obj}"
                    )
                except json.JSONDecodeError as e:
                    errors.append(
                        f"Error decoding 'postdata' in error file '{args.error}'. "
                        f"Content: {json_obj['request']['postdata']}. Error: {e}"
                    )
    except FileNotFoundError as e:
        errors.append(f"File not found: {args.error}. Please check the file path.")
    except ValueError as e:
        errors.append(str(e))

    # Load the response file into memory
    try:
        with open(args.response, "r") as response_f:
            response_data = parse_json_lines_to_memory(response_f, args.response)
            for json_obj in response_data:
                try:
                    msgnum = json_obj['message']['msgnum']
                    code = json_obj['response']['code']
                    body = json_obj['response']['body']
                    batch_index = json_obj['response']['batch_index']

                    if msgnum not in messages:
                        errors.append(
                            f"Message {msgnum} in response file '{args.response}' "
                            f"does not exist in the error file '{args.error}'."
                        )
                        continue

                    if messages[msgnum]['response']['status'] != code:
                        errors.append(
                            f"Status code mismatch for message {msgnum}:\n"
                            f"  Expected: {messages[msgnum]['response']['status']}\n"
                            f"  Found: {code}\n"
                            f"  Context: {json_obj}"
                        )

                    if messages[msgnum]['response']['message'] != body:
                        errors.append(
                            f"Message body mismatch for message {msgnum}:\n"
                            f"  Expected: {messages[msgnum]['response']['message']}\n"
                            f"  Found: {body}\n"
                            f"  Context: {json_obj}"
                        )

                    if messages[msgnum]['index'] != batch_index:
                        errors.append(
                            f"Batch index mismatch for message {msgnum}:\n"
                            f"  Expected: {messages[msgnum]['index']}\n"
                            f"  Found: {batch_index}\n"
                            f"  Context: {json_obj}"
                        )
                except KeyError as e:
                    errors.append(
                        f"Missing key {e} in response file '{args.response}'. "
                        f"Problematic JSON object: {json_obj}"
                    )
    except FileNotFoundError as e:
        errors.append(f"File not found: {args.response}. Please check the file path.")
    except ValueError as e:
        errors.append(str(e))

    # Report errors, limited by --max-errors
    if errors:
        print(f"Validation completed with {len(errors)} errors. Showing the first {min(len(errors), args.max_errors)} errors:\n")
        for error in errors[:args.max_errors]:
            print(f"- {error}")
        # Exit with non-zero code to indicate errors
        sys.exit(1)
    else:
        print("Validation completed successfully. No errors found.")
        # Exit with zero code to indicate success
        sys.exit(0)



# call this via "python[3] script name"

import sys
import os

lineLength = int(sys.argv[1])
linePrefix = "[{0:09d}] ".format(os.getpid())

logLine = sys.stdin.readline()
while logLine:
    logLine = logLine.strip()
    numRepeats = int(lineLength / len(logLine))

    lineToStdout = (linePrefix + "[stdout] " + logLine*numRepeats)[:lineLength]
    lineToStderr = (linePrefix + "[stderr] " + logLine*numRepeats)[:lineLength]

    sys.stdout.write(lineToStdout + "\n")
    sys.stderr.write(lineToStderr + "\n")

    # Flush stdout. In both Python 2 and Python 3, stdout is block-buffered when
    # redirected to a file/pipe. But be want each line to be written immediately,
    # because multiple processes are writing to the same pipe, and we do not want
    # lines to appear intermingled in the output file. (The flush will cause a
    # single 'write' syscall, since the size of the block buffer is generally
    # greater than PIPE_BUF.)
    sys.stdout.flush()

    # Flush stderr. This is not necessary in Python 2, since stderr is unbuffered
    # (and therefore each write will be written immediately and atomically to the
    # pipe). However, in Python 3 stderr is block-buffered when redirected to a
    # file/pipe. (Note: in future versions of Python 3, stderr could change to
    # line-buffered; see https://bugs.python.org/issue13601.)
    sys.stderr.flush()

    logLine = sys.stdin.readline()
